<?php
/**
 * Plugin Name: Finale - WooCommerce Sales Countdown Timer & Discount Plugin Lite
 * Plugin URI: https://xlplugins.com/finale-woocommerce-sales-countdown-timer-discount-plugin/
 * Description: Finale lets you create scheduled one time or recurring campaigns. It induces urgency with visual elements such as Countdown Timer and Counter Bar to motivate users to place an order.
 * Version: 1.3.2
 * Author: XLPlugins
 * Author URI: https://www.xlplugins.com
 * License: GPLv2 or later
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: wcct-finale-lite
 * XL: True
 *
 * Finale - WooCommerce Sales Countdown Timer & Discount Plugin Lite is free software.
 * You can redistribute it and/or modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Finale - WooCommerce Sales Countdown Timer & Discount Plugin Lite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Finale - WooCommerce Sales Countdown Timer & Discount Plugin Lite. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Finale-Lite
 * @Category Core
 * @author XLPlugins
 */
if (!defined('ABSPATH')) {
    exit;
}


if (!function_exists('wcct_finale_depenecy')) {

    /**
     * Function to check if wcct_finale pro version is loaded and activated or not?
     * @return bool True|False
     */
    function wcct_finale_depenecy() {

        $active_plugins = (array) get_option('active_plugins', array());

        if (is_multisite())
            $active_plugins = array_merge($active_plugins, get_site_option('active_sitewide_plugins', array()));


        return in_array('finale-woocommerce-sales-countdown-timer-discount-plugin/finale-woocommerce-sales-countdown-timer-discount-plugin.php', $active_plugins) || array_key_exists('finale-woocommerce-sales-countdown-timer-discount-plugin/finale-woocommerce-sales-countdown-timer-discount-plugin.php', $active_plugins);
    }

}


if (wcct_finale_depenecy()) {
    return;
}


if (!class_exists('WCCT_Core')) :

    class WCCT_Core {

        /**
         * @var WCCT_Core
         */
        public static $_instance = null;
        private static $_registered_entity = array('active' => array(), 'inactive' => array());

        /**
         * @var WCCT_Campaign
         */
        public $public;

        /**
         * @var WCCT_Appearance
         */
        public $appearance;

        /**
         * @var WCCT_Cart
         */
        public $cart;

        /**
         * @var WCCT_stock
         */
        public $stock;

        /**
         * @var WCCT_discount
         */
        public $discount;

        /**
         * @var WCCT_XL_Support
         */
        public $xl_support;

        /**
         * @var WCCT_Shortcode
         */
        public $shortcode;

        /**
         * @var bool Dependency check property
         */
        private $is_dependency_exists = true;

        public function __construct() {

            /**
             * Load important variables and constants
             */
            $this->define_plugin_properties();

            /**
             * Load dependency classes like woo-functions.php
             */
            $this->load_dependecies_support();

            /**
             * Run dependency check to check if dependency available
             */
            $this->do_dependecy_check();
            if ($this->is_dependency_exists) {

                /**
                 * Loads activation hooks
                 */
                $this->maybe_load_activation();

                /**
                 * Loads all the hooks
                 */
                $this->load_hooks();

                /**
                 * Initiates and loads XL start file
                 */
                $this->load_xl_core_classes();

                /**
                 * Include common classes
                 */
                $this->include_commons();

                /**
                 * Initialize common hooks and functions
                 */
                $this->initialize_common();

                /**
                 * Maybe load admin if admin screen
                 */
                $this->maybe_load_admin();
            }
        }

        public function define_plugin_properties() {
            /*             * ****** DEFINING CONSTANTS ********* */
            define("WCCT_VERSION", "1.3.2");
            define("WCCT_TEXTDOMAIN", "wcct-finale-lite");
            define("WCCT_FULL_NAME", "Finale - WooCommerce Sales Countdown Timer & Discount Plugin Lite");
            define("WCCT_PLUGIN_FILE", __FILE__);
            define("WCCT_PLUGIN_DIR", __DIR__);
            define("WCCT_PLUGIN_BASENAME", plugin_basename(__FILE__));
            define("WCCT_PURCHASE", 'xlplugin');
            define("WCCT_SHORT_SLUG", 'wcct');
        }

        public function load_dependecies_support() {
            /** Setting up WooCommerce Dependency Classes */
            require_once('woo-includes/woo-functions.php');
        }

        public function do_dependecy_check() {

            if (!wcct_is_woocommerce_active()) {

                add_action('admin_notices', array($this, 'wcct_wc_not_installed_notice'));
                $this->is_dependency_exists = false;
            }
        }

        public function maybe_load_activation() {

            /** Hooking action to the activation */
            register_activation_hook(__FILE__, array($this, 'wcct_activation'));
        }




        public function load_hooks() {

            /** Initializing Functionality */
            add_action('plugins_loaded', array($this, 'wcct_init'), 0);

            add_action('plugins_loaded', array($this, 'wcct_register_classes'), 1);
            /** Initialize Localization */
            add_action('init', array($this, 'wcct_init_localization'));

            /** Redirecting Plugin to the settings page after activation */
            add_action('activated_plugin', array($this, 'wcct_settings_redirect'));
        }

        public function load_xl_core_classes() {

            /** Setting Up XL Core */
            require_once('start.php');
        }

        public function include_commons() {
            /** Loading Common Class */
            require 'includes/wcct-common.php';
            require 'includes/wcct-xl-support.php';
        }

        public function initialize_common() {
            /** Firing Init to init basic Functions */
            WCCT_Common::init();
        }

        public function maybe_load_admin() {
            /* ----------------------------------------------------------------------------*
             * Dashboard and Administrative Functionality
             * ---------------------------------------------------------------------------- */
            if (is_admin() && (!defined('DOING_AJAX') || !DOING_AJAX)) {
                require_once(plugin_dir_path(WCCT_PLUGIN_FILE) . 'admin/wcct-admin.php');
            }
			/**
			 * Loading upsell class
			 */
			if ( is_admin() ) {
				include_once plugin_dir_path( WCCT_PLUGIN_FILE ) . 'admin/includes/upsell/class-wcct-upsell.php';
			}
        }

        public function wcct_register_classes() {
            $load_classes = self::get_registered_class();

            if (is_array($load_classes) && count($load_classes) > 0) {
                foreach ($load_classes as $access_key => $class) {

                    $this->$access_key = $class::get_instance();
                }
            }
        }

        public static function get_registered_class() {
            return self::$_registered_entity['active'];
        }

        public static function register($shortName, $class, $overrides = null) {

            //Ignore classes that have been marked as inactive
            if (in_array($class, self::$_registered_entity['inactive'])) {
                return;
            }

            //Mark classes as active. Override existing active classes if they are supposed to be overridden
            $index = array_search($overrides, self::$_registered_entity['active']);
            if ($index !== false) {
                self::$_registered_entity['active'][$index] = $class;
            } else {
                self::$_registered_entity['active'][$shortName] = $class;
            }

            //Mark overridden classes as inactive.
            if (!empty($overrides)) {
                self::$_registered_entity['inactive'][] = $overrides;
            }
        }

        /**
         * @return null|WCCT_Core
         */
        public static function get_instance() {
            if (self::$_instance == null) {
                self::$_instance = new self;
            }
            return self::$_instance;
        }

        /** Triggering activation initialization */
        public function wcct_activation() {
            WCCT_Admin::handle_activation();
        }

        public function wcct_init_localization() {
            load_plugin_textdomain(WCCT_TEXTDOMAIN, FALSE, plugin_basename(dirname(__FILE__)) . '/languages');
        }

        /**
         * Added redirection on plugin activation
         * @param $plugin
         */
        public function wcct_settings_redirect($plugin) {
            if (wcct_is_woocommerce_active() && class_exists("WooCommerce")) {
                if ($plugin == plugin_basename(__FILE__)) {
                    wp_redirect(add_query_arg(array('page' => 'wc-settings', 'tab' => WCCT_Common::get_wc_settings_tab_slug()), admin_url('admin.php')));
                    exit;
                }
            }
        }

        /**
         * Checking WooCommerce dependency and then loads further
         * @return bool false on failure
         */
        public function wcct_init() {


            if (wcct_is_woocommerce_active() && class_exists("WooCommerce")) {

                global $woocommerce;
                if (!version_compare($woocommerce->version, WCCT_VERSION, ">=")) {
                    add_action('admin_notices', array($this, 'wcct_wc_version_check_notice'));
                    return false;
                }

                if (is_user_logged_in() && current_user_can('administrator') && isset($_GET['wcct_disable']) && $_GET['wcct_disable'] == 'yes') {
                    return;
                }

                require 'includes/wcct-logging.php';

                if (!is_admin()) {
                    require 'includes/wcct-triggers-data.php';
                    require 'includes/wcct-campaign-public.php';
                    require 'includes/wcct-appearance.php';
                    require 'includes/wcct-cart.php';
                    require 'includes/wcct-discount.php';
                    require 'includes/wcct-stock.php';
                    require 'includes/wcct-themes-helper.php';
                }
            }
        }

        /*         * ******** REGISTERING NOTICES ******************* */

        public function wcct_wc_version_check_notice() {
            ?>
            <div class="error">
                <p><?php _e("WooCommerce Url require WooCommerce version " . WCCT_VERSION . " or greater", WCCT_TEXTDOMAIN); ?></p>
            </div>
            <?php
        }

        public function wcct_wc_not_installed_notice() {
            ?>
            <div class="error">
                <p><?php _e(WCCT_FULL_NAME . ": WooCommerce not Activated or not Installed.", WCCT_TEXTDOMAIN); ?></p>
            </div>
            <?php
        }

    }

    endif;


/**
 * Global Common function to load all the classes
 * @param bool $debug
 * @return WCCT_Core
 */
if (!function_exists('WCCT_Core')) {

    function WCCT_Core($debug = false) {
        return WCCT_Core::get_instance();
    }

}


$GLOBALS['WCCT_Core'] = WCCT_Core();










