<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Class WCCT_discount
 * @package Finale-Lite
 * @author XlPlugins
 */
class WCCT_discount
{

    public static $_instance = null;

    public function __construct()
    {


        global $woocommerce;

        if (version_compare($woocommerce->version, 3.0, ">=")) {

            //simple
            add_filter("woocommerce_product_get_price", array($this, "wcct_trigger_get_price"), 999, 2);
            add_filter("woocommerce_product_get_sale_price", array($this, "wcct_trigger_get_sale_price"), 999, 2);

            add_filter("woocommerce_product_variation_get_price", array($this, "wcct_trigger_get_price"), 999, 2);
            add_filter("woocommerce_product_variation_get_sale_price", array($this, "wcct_trigger_get_sale_price"), 999, 2);


        } else {

            add_filter("woocommerce_get_price", array($this, "wcct_trigger_get_price"), 10, 2);
            add_filter("woocommerce_get_sale_price", array($this, "wcct_trigger_get_sale_price"), 999, 2);

        }
        add_filter('woocommerce_product_is_on_sale', array($this, 'wcct_maybe_mark_product_having_sale'), 999, 2);


        add_filter("woocommerce_variation_prices", array($this, "wcct_change_price_ranges"), 900, 3);


    }

    public static function get_instance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self;
        }
        return self::$_instance;
    }

    public function wcct_trigger_get_price($get_price, $product_global)
    {


        $is_skip = apply_filters("wcct_skip_discounts",FALSE,$get_price, $product_global );

        wcct_force_log("Product id {$product_global->get_id()} : " . __FUNCTION__ . " Before Price: " . $get_price);
        if ($get_price == "") {
            return $get_price;
        }

        if($is_skip === TRUE) {
            return $get_price;
        }

        if (in_array($product_global->get_type(), WCCT_Common::get_sale_compatible_league_product_types())) {


            $get_price = $this->wcct_trigger_create_price($get_price, $product_global);
        }
        wcct_force_log("Product id {$product_global->get_id()} : " . __FUNCTION__ . " After Price: " . $get_price);

        return $get_price;
    }

    public function wcct_trigger_create_price($sale_price, $product_global, $mode = "basic", $regular_price = FALSE)
    {
        global $post;


        $type = $product_global->get_type();
        $parentId = WCCT_Core()->public->wcct_get_product_parent_id($product_global);
        if (WCCT_Core()->public->wcct_restrict_for_booking_oth($parentId)) {
            return $sale_price;
        }
        $tempId = $parentId;
        $data = WCCT_Core()->public->get_single_campaign_pro_data($tempId);


        if (empty($data)) {
            wcct_force_log(" terminating " . __FUNCTION__ . " For Product" . $tempId);
            return $sale_price;
        }

        if (isset($data["deals"]) && !is_array($data["deals"])) {
            return $sale_price;
        }


        WCCT_Core()->public->wcct_get_product_obj($tempId);
        $deals = $data["deals"];
        $deals_overide = $deals;
        if ($regular_price === FALSE) {
            $regular_price = $product_global->get_regular_price();
        }

        if (!is_array($deals_overide)) {

            return $sale_price;
        }

        $check_sale = get_post_meta($product_global->get_id(), "_sale_price", true); // we are fetching sale price from db using get_post_meta otherwise will stick in loop

        if ($check_sale >= '0' && $deals_overide["override"] == 1) {
            $sale_price = (float)$sale_price;
            return $sale_price;
        }
        $deal = $deals_overide;
        $deal_amount = (int)isset($deal["deal_amount"]) ? $deal["deal_amount"] : 0;
        $deal_amount = (int)$deal_amount;

        if ($deal_amount >= 0) {
            if ($deal["type"] == "fixed_price") {
                $deal_amount = apply_filters("wcct_deal_amount_fixed_amount_{$type}", $deal_amount, $product_global, $data);

                if ($mode == "sale" && $deal_amount == '0') {
                    return "";
                }
                $set_sale_price = $regular_price - $deal_amount;

                if ($set_sale_price >= 0) {
                    $sale_price = $set_sale_price;
                } else {
                    $sale_price = 0;
                }
            } else {
                $deal_amount = apply_filters("wcct_deal_amount_percentage_{$type}", $deal_amount, $product_global, $data);

                if ($mode == "sale" && $deal_amount == '0') {
                    return "";
                }
                $set_sale_price = $regular_price - ($regular_price * ($deal_amount / 100));
                if ($set_sale_price >= 0) {
                    $sale_price = $set_sale_price;
                } else {
                    $sale_price = 0;
                }
            }
        } else {

            return $sale_price;
        }

        return $sale_price;
    }

    public function wcct_trigger_get_sale_price($sale_price, $product_global)
    {
        wcct_force_log("Product id {$product_global->get_id()} : " . __FUNCTION__ . " Before Price: " . $sale_price);

        if (in_array($product_global->get_type(), WCCT_Common::get_sale_compatible_league_product_types())) {
            $sale_price = $this->wcct_trigger_create_price($sale_price, $product_global, 'sale');
        }

        wcct_force_log("Product id {$product_global->get_id()} : " . __FUNCTION__ . " After Price: " . $sale_price);
        return $sale_price;
    }

    public function wcct_trigger_create_sale_variation($sale_price, $variation, $product_global)
    {
        return $this->wcct_trigger_create_price($sale_price, $variation, "sale");
    }


    public function wcct_date_on_sale_from($sale_from_date, $product_global) {
        global $post;

        $parentId = WCCT_Core()->public->wcct_get_product_parent_id($product_global);

        if (WCCT_Core()->public->wcct_restrict_for_booking_oth($parentId)) {
            return $sale_from_date;
        }
        $tempId = $parentId;
        $data = WCCT_Core()->public->get_single_campaign_pro_data($parentId);

        if (isset($data["deals"]) && count($data["deals"]) > 0) {
            $deals = $data["deals"];
            if (isset($deals["override"]) && $deals["override"] == "1" && is_object($sale_from_date)) {
                return $sale_from_date;
            }



            $sale_start_date = (int) $data['deals']["start_time"];
            //sale end condition custom check
//            $sale_start_date = $sale_start_date - 60 * 60 * 24;
            $timezone = new DateTimeZone(WCCT_Common::wc_timezone_string());
            if ($sale_from_date instanceof WC_DateTime) {
                $sale_from_date->setTimezone($timezone);
                $sale_from_date->setTimestamp($sale_start_date);
            } else {
                $sale_from_date = new WC_DateTime();
                $sale_from_date->setTimezone($timezone);
                $sale_from_date->setTimestamp($sale_start_date);
            }






        }

        return $sale_from_date;
    }

    //Events

    public function wcct_date_on_sale_to($sale_from_to, $product_global) {
        global $post;

        $parentId = WCCT_Core()->public->wcct_get_product_parent_id($product_global);
        if (WCCT_Core()->public->wcct_restrict_for_booking_oth($parentId)) {
            return $sale_from_to;
        }
        $data = WCCT_Core()->public->get_single_campaign_pro_data($parentId);

        if (isset($data["deals"]) && count($data["deals"]) > 0) {
            $deals = $data["deals"];
            if (isset($deals["override"]) && $deals["override"] == "1" && is_object($sale_from_to)) {
                return $sale_from_to;
            }


            $sale_end_date = (int)$data['deals']["end_time"];
            $timezone = new DateTimeZone(WCCT_Common::wc_timezone_string());
            if ($sale_from_to instanceof WC_DateTime) {
                $sale_from_to->setTimezone($timezone);
                $sale_from_to->setTimestamp($sale_end_date);
            } else {
                $sale_from_to = new WC_DateTime();
                $sale_from_to->setTimezone($timezone);
                $sale_from_to->setTimestamp($sale_end_date);

            }
        }





        return $sale_from_to;
    }

    public function wcct_change_price_ranges($priceRanges, $productThis, $display)
    {


        if ($display === FALSE)
            return $priceRanges;

        $prices = array();
        $regular_prices = array();
        $sale_prices = array();
        if (is_array($priceRanges) && count($priceRanges) > 0) {

            foreach ($priceRanges['regular_price'] as $key => $val) {

                $regular_prices[$key] = $val;

            }


            $prices = $this->wcct_set_variation_price($priceRanges['price'], "basic", $regular_prices);


            $sale_prices = $this->wcct_set_variation_price($priceRanges['price'], "basic", $regular_prices);


        }
        asort($prices);
        asort($regular_prices);
        asort($sale_prices);

//
        $priceRanges = array(
            'price' => $prices,
            'regular_price' => $regular_prices,
            'sale_price' => $sale_prices,
        );


        return $priceRanges;
    }

    public function wcct_set_variation_price($input, $type = "basic", $regular_price = FALSE)
    {
        if (is_array($input)) {

            foreach ($input as $k => $price) {
                $product = null;

                $product = WCCT_Core()->public->wcct_get_product_obj($k);

                //formatting the prices as per WC is doing so that further comparison can take place between reg price and sale price to detect is on sale

                $input[$k] = wc_format_decimal($this->wcct_trigger_create_price($price, $product, $type, ($regular_price !== FALSE) ? $regular_price[$k] : FALSE) . '.00', wc_get_price_decimals());
            }

        }
        return $input;
    }


    public function wcct_maybe_mark_product_having_sale($bool, $product)
    {


        if (in_array($product->get_type(), WCCT_Common::get_variable_league_product_types())) {

            $priceRanges = $product->get_variation_prices();
            $prices = array();
            $regular_prices = array();
            $sale_prices = array();
            if (is_array($priceRanges) && count($priceRanges) > 0) {


                foreach ($priceRanges['regular_price'] as $key => $val) {

                    $regular_prices[$key] = $val;

                }


                $prices = $this->wcct_set_variation_price($priceRanges['price'], "basic", $regular_prices);


                $sale_prices = $this->wcct_set_variation_price($priceRanges['price'], "basic", $regular_prices);


            }
            asort($prices);
            asort($regular_prices);
            asort($sale_prices);


            $priceRanges = array(
                'price' => $prices,
                'regular_price' => $regular_prices,
                'sale_price' => $sale_prices,
            );

            if ($priceRanges['regular_price'] !== $priceRanges['sale_price'] && $priceRanges['sale_price'] === $priceRanges['price']) {
                $bool = true;
            }


            return $bool;
        }

        return $bool;
    }


}

if (class_exists("WCCT_discount")) {
    WCCT_Core::register("discount", "WCCT_discount");
}