<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}


/**
 * Class WCCT_stock
 * @package Finale-Lite
 * @author XlPlugins
 */
class WCCT_stock
{

    public static $_instance = null;

    public function __construct()
    {
        global $woocommerce;

        if (version_compare($woocommerce->version, 3.0, ">=")) {

            add_filter('woocommerce_product_get_manage_stock', array($this, 'wcct_modify_manage_stock'), 10, 2);
            add_filter('woocommerce_product_variation_get_manage_stock', array($this, 'wcct_modify_manage_stock'), 10, 2);

            add_filter('woocommerce_product_get_stock_quantity', array($this, 'wcct_manage_stock_qty'), 10, 2);

            add_filter('woocommerce_product_variation_get_stock_quantity', array($this, 'wcct_manage_stock_qty'), 10, 2);
            add_filter("woocommerce_product_is_in_stock", array($this, "wcct_woocommerce_product_is_in_stock"), 10, 2);
            add_action('woocommerce_variation_is_in_stock', array($this, 'wcct_woocommerce_product_is_in_stock'), 10);

        } else {
            add_filter('woocommerce_get_stock_quantity', array($this, 'wcct_manage_stock_qty'), 10, 2);
            add_filter('woocommerce_variation_get_stock_quantity', array($this, 'wcct_manage_stock_qty'), 10, 2);

            add_action('woocommerce_variation_is_in_stock', array($this, 'wcct_woocommerce_product_is_in_stock'), 10);
            add_filter("woocommerce_product_is_in_stock", array($this, "wcct_woocommerce_product_is_in_stock"), 10);
        }

        add_action('woocommerce_product_backorders_allowed', array($this, 'wcct_backorders_allowed'), 10, 3);

    }

    public static function get_instance()
    {
        if (self::$_instance == null) {
            self::$_instance = new self;
        }
        return self::$_instance;
    }

    public function wcct_woocommerce_product_is_in_stock($status, $product_obj = '') {
        // checking on cart
        global $product;

        if(WCCT_Common::$is_executing_rule){
            return $status;
        }

        if (!is_object($product_obj) && $product_obj == '') {
            // WC 2.6 or older
            $my_product = $product;
            if (WCCT_Core()->cart->cart_product_id != 0) {
                $my_product_id = WCCT_Core()->cart->cart_product_id;
                $my_product = WCCT_Core()->public->product_obj[$my_product_id];
            } else {
                if (isset($_REQUEST["add-to-cart"])) {
                    $my_product_id = $_REQUEST["add-to-cart"];
                    $my_product = wc_get_product($my_product_id);
                    WCCT_Core()->public->product_obj[$my_product_id] = $my_product;
                } elseif (isset($_REQUEST["product_id"])) {
                    $my_product_id = $_REQUEST["product_id"];
                    $my_product = wc_get_product($my_product_id);
                    WCCT_Core()->public->product_obj[$my_product_id] = $my_product;
                } elseif (is_object($my_product)) {

                    $my_product_id = $my_product->get_id();
                    WCCT_Core()->public->product_obj[$my_product_id] = $my_product;
                } else {
                    $my_product_id = 0;
                }
            }
        } else {
            // i.e. WC 3.0
            $my_product_id = WCCT_Core()->public->wcct_get_product_parent_id($product_obj); // parent id in case of variable
            $my_product = WCCT_Core()->public->wcct_get_product_obj($my_product_id);

        }


        if ($my_product_id > 0) {


            if (WCCT_Core()->public->wcct_restrict_for_booking_oth($my_product_id)) {
                return $status;
            }
        } else {
            return $status;
        }



        $single_data = WCCT_Core()->public->get_single_campaign_pro_data($my_product_id);

        if ($my_product && $my_product->get_type() == "simple") {

            if (isset($single_data["goals"]) && count($single_data["goals"]) > 0) {
                $goals = $single_data["goals"];

                if ($goals['type'] == "custom") {

                    if ($goals["allow_backorder"] == "yes") {
                        $status = true;
                    } else {
                        $wcct_campaign_event_product_stock_state = "_wcct_goaldeal_stock_{$goals['campaign_id']}_{$goals['start_timestamp']}_{$goals['end_timestamp']}";

                        $get_product_state_meta = get_post_meta($my_product_id, $wcct_campaign_event_product_stock_state, true);

                        if ($get_product_state_meta !== "") {
                            $status =  true;
                        }
                    }
                }
            }
        } elseif ($my_product && $my_product->get_type() == "variable") {
            if (isset($single_data["goals"]) && count($single_data["goals"]) > 0) {
                $goals = $single_data["goals"];
                if (isset($goals["start_timestamp"])) {
                    if ($goals['type'] == "custom") {

                        if ($goals["allow_backorder"] == "yes") {
                            $status = true;
                        } else {
                            $wcct_campaign_event_product_stock_state = "_wcct_goaldeal_stock_{$goals['campaign_id']}_{$goals['start_timestamp']}_{$goals['end_timestamp']}";

                            $get_product_state_meta = get_post_meta($my_product->get_id(), $wcct_campaign_event_product_stock_state, true);

                            if ($get_product_state_meta !== "") {
                                $status =  true;
                            }
                        }
                    }
                }
            }
        }
        $status_string = ($status) ?"yes":"no";


        wcct_force_log("Product id {$my_product_id} : function  wcct_woocommerce_product_is_in_stock " .$status_string  );
        WCCT_Core()->cart->cart_product_id = 0;

        return $status;
    }

    public function wcct_backorders_allowed($status, $pid, $product = null) {


        if($product == null) {

            $product = WCCT_Core()->public->wcct_get_product_obj($pid);
        }
        if(WCCT_Common::$is_executing_rule){
            return $status;
        }
        if (WCCT_Core()->public->wcct_restrict_for_booking_oth($pid)) {
            return $status;
        }

        $pid = WCCT_Core()->public->wcct_get_product_parent_id($product);
        $single_data = WCCT_Core()->public->get_single_campaign_pro_data($pid);

        if ($product->get_type() == "simple") {
            if (isset($single_data["goals"]) && count($single_data["goals"]) > 0) {
                $goals = $single_data["goals"];
                if ($status == "yes" && $goals['type'] == "custom") {
                    $status = false;
                }
            }
        }

        if ($product->get_type() == "variation") {
            if (isset($single_data["goals"]) && count($single_data["goals"]) > 0) {
                $goals = $single_data["goals"];
                if (isset($goals["start_timestamp"]) && $status == "yes" && $goals['type'] == "custom") {
                    $status = true;
                }
            }
        }

        $backorder_status = ($status)?"yes":"no";

        wcct_force_log("Product id {$pid} : " . __FUNCTION__ . " " . $backorder_status);
        return $status;
    }

    /**
     * @param $qty
     * @param WC_Product $product
     * @return bool
     */
    public function wcct_manage_stock_qty($qty, $product) {


        if(WCCT_Common::$is_executing_rule){
            return $qty;
        }
        $product_id = WCCT_Core()->public->wcct_get_product_parent_id($product);

        $single_data = WCCT_Core()->public->get_single_campaign_pro_data($product_id);


        if (empty($single_data)) {
            return $qty;
        }
        $available_qty = false;


        $get_goal_object = WCCT_Core()->public->wcct_get_goal_object($single_data['goals'], $product_id);

        if (!empty($get_goal_object)) {
            $available_qty = $get_goal_object['quantity'] - $get_goal_object['sold_out'];
        }

        if ($available_qty !== false && $get_goal_object['type'] == "custom") {
            $qty = $available_qty;
        }


        return $qty;
    }

    /**
     * @hooked over `woocommerce_product_get_manage_stock` | `woocommerce_product_get_manage_stock`
     * @param boolean $bool is_mnaging product stock
     * @param WC_Product $product product Object
     * @return bool modified stock management status
     * Specifically for WC 3.0 or greater
     */
    public function wcct_modify_manage_stock( $bool , $product)  {



        if(WCCT_Common::$is_executing_rule){
            return $bool;
        }


        $product_id = WCCT_Core()->public->wcct_get_product_parent_id($product);

        $single_data = WCCT_Core()->public->get_single_campaign_pro_data($product_id);


        if (empty($single_data)) {
            return $bool;
        }
        $available_qty = false;


        $get_goal_object = WCCT_Core()->public->wcct_get_goal_object($single_data['goals'], $product_id);

        if (!empty($get_goal_object)) {
            $available_qty = $get_goal_object['quantity'] - $get_goal_object['sold_out'];
        }

        if ($available_qty !== false && $get_goal_object['type'] == "custom") {
            $bool = true;
        }


        return $bool;
    }
}

if (class_exists("WCCT_stock")) {
    WCCT_Core::register("stock", "WCCT_stock");
}