<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Class WCCT_XL_Support
 * @package Finale-Lite
 * @author XlPlugins
 */
class WCCT_XL_Support {

    public static $_instance = null;
    public $full_name = WCCT_FULL_NAME;
    public $is_license_needed = false;
    public $license_instance;
    public $expected_url;

    /**
     * Options Page title
     * @var string
     */
    protected $title = 'Sales Triggers';
    protected $slug = 'wcct';

    public function __construct() {
        $this->expected_url = admin_url('admin.php?page=xlplugins');

        /**
         * XL CORE HOOKS
         */
        add_filter("xl_optin_notif_show", array($this, 'wcct_xl_show_optin_pages'), 10, 1);


        add_action('admin_init', array($this, 'wcct_xl_expected_slug'), 9.1);
        add_action('maybe_push_optin_notice_state_action', array($this, 'wcct_try_push_notification_for_optin'), 10);

        add_action('admin_init', array($this, 'modify_api_args_if_wcct_dashboard'), 20);
        add_filter('extra_plugin_headers', array($this, 'extra_woocommerce_headers'));

        add_filter('add_menu_classes', array($this, 'modify_menu_classes'));
        add_action('admin_init', array($this, 'wcct_xl_parse_request_and_process'), 15);


        add_action('xl_licenses_submitted', array($this, 'process_licensing_form'));
        add_action('xl_deactivate_request', array($this, 'maybe_process_deactivation'));

        add_filter('xl_dashboard_tabs', array($this, 'wcct_modify_tabs'), 999, 1);
        add_filter('xl_after_license_table_notice', array($this, 'wcct_after_license_table_notice'), 999, 1);

//        add_action('xl_support_right_area', array($this, 'wcct_add_right_section_on_xlpages'));
//        add_action('xl_licenses_right_content', array($this, 'wcct_add_right_section_on_xlpages'));
        add_action('wcct_options_page_right_content', array($this, 'wcct_options_page_right_content'), 10);

        add_action('admin_menu', array($this, 'add_menus'), 80.1);
        add_action('admin_menu', array($this, 'add_wcct_menu'), 85.2);


        add_action('xl_tabs_modal_licenses', array($this, 'schedule_license_check'), 1);
        add_filter('xl_uninstall_reasons', array($this, 'modify_uninstall_reason'));

        add_filter('xl_uninstall_reason_threshold_' . WCCT_PLUGIN_BASENAME, function () {
            return 4;
        });

    }

    /**
     * @return null|WCCT_XL_Support
     */
    public static function get_instance() {
        if (self::$_instance == null) {
            self::$_instance = new self;
        }
        return self::$_instance;
    }

    public function wcct_xl_show_optin_pages($is_show) {
        return true;
    }

    public function wcct_xl_expected_slug() {
        if (isset($_GET['page']) && ($_GET['page'] == "xlplugins" || $_GET['page'] == "xlplugins-support" || $_GET['page'] == "xlplugins-addons")) {
            XL_dashboard::set_expected_slug($this->slug);
        }
        XL_dashboard::set_expected_url($this->expected_url);

        /**
         * Pushing notifications for invalid licenses found in ecosystem
         */
        $licenses = XL_licenses::get_instance()->get_data();
        $invalid_licenses = array();
        if ($licenses && count($licenses) > 0) {
            foreach ($licenses as $key => $license) {
                if ($license['product_status'] == "invalid") {
                    $invalid_licenses[] = $license['plugin'];
                }
            }
        }


        if (!XL_admin_notifications::has_notification('license_needs_attention') && count($invalid_licenses) > 0) {
            $license_invalid_text = sprintf(__('<p>You are <strong>not receiving</strong> Latest Updates, New Features, Security Updates &amp; Bug Fixes for <strong>%s</strong>. <a href="%s">Click Here To Fix This</a>.</p>', WCCT_TEXTDOMAIN), implode(",", $invalid_licenses), add_query_arg(array('tab' => 'licenses'), $this->expected_url));

            XL_admin_notifications::add_notification(array('license_needs_attention' => array(
                    'type' => 'error',
                    'is_dismissable' => false,
                    'content' => $license_invalid_text,
            )));
        }
    }

    public function wcct_metabox_always_open($classes) {
        if (($key = array_search('closed', $classes)) !== false) {
            unset($classes[$key]);
        }
        return $classes;
    }

    public function modify_api_args_if_wcct_dashboard() {
        if (XL_dashboard::get_expected_slug() == $this->slug) {
            add_filter('xl_api_call_agrs', array($this, 'modify_api_args_for_gravityxl'));
            XL_dashboard::register_dashboard(array('parent' => array('woocommerce' => "WooCommerce Add-ons"), 'name' => $this->slug));
        }
    }

    public function xlplugins_page() {
        if (!isset($_GET['tab'])) {

            $licenses = apply_filters('xl_plugins_license_needed', array());

            if (empty($licenses)) {
                XL_dashboard::$selected = "support";
            }else{
                XL_dashboard::$selected = "licenses";
            }




        }
        XL_dashboard::load_page();
    }

    public function xlplugins_support_page() {
        if (!isset($_GET['tab'])) {
            XL_dashboard::$selected = "support";
        }
        XL_dashboard::load_page();
    }

    public function xlplugins_plugins_page() {
        XL_dashboard::$selected = "plugins";
        XL_dashboard::load_page();
    }

    public function modify_api_args_for_gravityxl($args) {
        if (isset($args['edd_action']) && $args['edd_action'] == "get_xl_plugins") {
            $args['attrs']['tax_query'] = array(
                array(
                    'taxonomy' => 'xl_edd_tax_parent',
                    'field' => 'slug',
                    'terms' => 'woocommerce',
                    'operator' => 'IN'
                )
            );
        }
        $args['purchase'] = WCCT_PURCHASE;
        return $args;
    }

    public function wcct_try_push_notification_for_optin() {
        if (!XL_admin_notifications::has_notification('xl_optin_notice')) {
            XL_admin_notifications::add_notification(array('xl_optin_notice' => array(
                    'type' => 'info',
                    'content' => sprintf('
                        <p>We\'re always building new features into <a target="_blank" href=\'%s\'>Finale</a>, Play a part in shaping the future of Finale and in turn benefit from new conversion-boosting updates.</p>
                        <p>Simply by allowing us to learn about your plugin usage. No sensitive information will be passed on to us. It\'s all safe & secure to say YES.</p>
                        <p><a href=\'%s\' class=\'button button-primary\'>Yes, I want to help</a> <a href=\'%s\' class=\'button button-secondary\'>No, I don\'t want to help</a> <a style="float: right;" target="_blank" href=\'%s\'>Know More</a></p> ', esc_url("https://xlplugins.com/finale-woocommerce-sales-countdown-timer-discount-plugin/?utm_source=wpplugin&utm_campaign=finale&utm_medium=text&utm_term=optin"), esc_url(wp_nonce_url(add_query_arg(array('xl-optin-choice' => 'yes', 'ref' => filter_input(INPUT_GET, 'page'))), 'xl_optin_nonce', '_xl_optin_nonce')), esc_url(wp_nonce_url(add_query_arg('xl-optin-choice', 'no'), 'xl_optin_nonce', '_xl_optin_nonce')), esc_url("https://xlplugins.com/data-collection-policy/?utm_source=wpplugin&utm_campaign=finale&utm_medium=text&utm_term=optin"))
                )
                    )
            );
        }
    }

    /**
     * Adding XL Header to tell wordpress to read one extra params while reading plugin's header info/. <br/>
     * Hooked over `extra_plugin_headers`
     * @since 1.0.0
     * @param array $headers already registered arrays
     * @return type
     */
    public function extra_woocommerce_headers($headers) {
        array_push($headers, 'XL');
        return $headers;
    }

    public function modify_menu_classes($menu) {
        return $menu;
    }

    public function wcct_xl_parse_request_and_process() {
        $instance_support = XL_Support::get_instance();

        if ($this->slug == XL_dashboard::get_expected_slug() && isset($_POST['xl_submit_support'])) {

            if (filter_input(INPUT_POST, 'choose_addon') == "" || filter_input(INPUT_POST, 'comments') == "") {
                $instance_support->validation = false;
                XL_admin_notifications::add_notification(array('support_request_failure' => array(
                        'type' => 'error',
                        'is_dismissable' => true,
                        'content' => __('<p> Unable to submit your request.All fields are required. Please ensure that all the fields are filled out.</p>', WCCT_TEXTDOMAIN),
                )));
            } else {
                $instance_support->xl_maybe_push_support_request($_POST);
            }
        }
    }

    public function process_licensing_form($posted_data) {
        if (isset($posted_data['license_keys'][WCCT_PLUGIN_BASENAME])) {
            $shortname = $this->edd_slugify_module_name($this->full_name);
            update_option('xl_licenses_' . $shortname, $posted_data['license_keys'][WCCT_PLUGIN_BASENAME], false);
            $this->license_instance->activate_license($posted_data['license_keys'][WCCT_PLUGIN_BASENAME]);
        }
    }

    /**
     * License management helper function to create a slug that is friendly with edd
     * @param type $name
     * @return type
     */
    public function edd_slugify_module_name($name) {
        return preg_replace('/[^a-zA-Z0-9_\s]/', '', str_replace(' ', '_', strtolower($name)));
    }

    /**
     * Validate is it is for email product deactivation
     * @param type $posted_data
     */
    public function maybe_process_deactivation($posted_data) {
        if (isset($posted_data['filepath']) && $posted_data['filepath'] == WCCT_PLUGIN_BASENAME) {
            $this->license_instance->deactivate_license();
            wp_safe_redirect('admin.php?page=' . $posted_data['page'] . "&tab=" . $posted_data['tab']);
        }
    }

    public function wcct_modify_tabs($tabs) {
        if ($this->slug == XL_dashboard::get_expected_slug()) {
            return array();
        }
        return $tabs;
    }

    public function wcct_after_license_table_notice($notice) {
        return 'Note: You need to have a valid license key to receiving updates for these plugins. Click here to get your <a href="https://xlplugins.com/manage-licenses/" target="_blank">License Keys</a>.';
    }

    public function wcct_add_right_section_on_xlpages() {
        
    }

    /**
     * Adding WooCommerce sub-menu for global options
     */
    public function add_menus() {

        if (!XL_dashboard::$is_core_menu) {

            add_menu_page(__('XLPlugins', WCCT_TEXTDOMAIN), __('XLPlugins', WCCT_TEXTDOMAIN), 'manage_woocommerce', 'xlplugins', array($this, 'xlplugins_page'), '', '59.5');


            $licenses = apply_filters('xl_plugins_license_needed', array());
            if (!empty($licenses)) {
                add_submenu_page('xlplugins', __('Licenses', WCCT_TEXTDOMAIN), __('License', WCCT_TEXTDOMAIN), 'manage_woocommerce', 'xlplugins');
            }


            XL_dashboard::$is_core_menu = true;
        }
    }

    public function add_wcct_menu() {
        add_submenu_page('xlplugins', WCCT_FULL_NAME, 'Finale Lite', 'manage_woocommerce', 'admin.php?page=wc-settings&tab=' . WCCT_Common::get_wc_settings_tab_slug() . '', false);
    }

    public function wcct_options_page_right_content() {
        $get_instance_access_link = add_query_arg(array(
            'utm_source' => 'finale-lite',
            'utm_medium' => 'sidebar',
            'utm_campaign' => 'ebook-download',
            'utm_term' => '13-promotional-campaign-ideas',
                ), 'https://xlplugins.com/high-converting-woocommerce-sales-promotional-campaigs/');
        $go_pro_link = add_query_arg(array(
            'utm_source' => 'finale-lite',
            'utm_medium' => 'sidebar',
            'utm_campaign' => 'plugin-resource',
            'utm_term' => 'buy_now',
                ), 'https://xlplugins.com/lite-to-pro-upgrade-page/');
        $demo_link = add_query_arg(array(
            'utm_source' => 'finale-lite',
            'utm_medium' => 'sidebar',
            'utm_campaign' => 'plugin-resource',
            'utm_term' => 'demo',
                ), 'http://demo.xlplugins.com/finale/');
        $support_link = add_query_arg(array(
            'pro' => 'finale',
            'utm_source' => 'finale-lite',
            'utm_medium' => 'sidebar',
            'utm_campaign' => 'plugin-resource',
            'utm_term' => 'support',
                ), 'https://xlplugins.com/support/');
        $documentation_link = add_query_arg(array(
            'utm_source' => 'finale-lite',
            'utm_medium' => 'sidebar',
            'utm_campaign' => 'plugin-resource',
            'utm_term' => 'documentation',
                ), 'https://xlplugins.com/documentation/');

        $other_products = array();
        if (!class_exists('WCCT_Core')) {
            $finale_link = add_query_arg(array(
                'utm_source' => 'finale-lite',
                'utm_medium' => 'sidebar',
                'utm_campaign' => 'other-products',
                'utm_term' => 'finale',
                    ), 'https://xlplugins.com/finale-woocommerce-sales-countdown-timer-discount-plugin/');
            $other_products['finale'] = array(
                'image' => 'finale.png',
                'link' => $finale_link,
                'head' => 'Finale WooCommerce Sales Countdown Timer',
                'desc' => 'Run Urgency Marketing Campaigns On Your Store And Move Buyers to Make A Purchase',
            );
        }
        if (!defined('WCST_SLUG')) {
            $sales_trigger_link = add_query_arg(array(
                'utm_source' => 'finale-lite',
                'utm_medium' => 'sidebar',
                'utm_campaign' => 'other-products',
                'utm_term' => 'sales-trigger',
                    ), 'https://xlplugins.com/woocommerce-sales-triggers/');
            $other_products['sales_trigger'] = array(
                'image' => 'sales-trigger.png',
                'link' => $sales_trigger_link,
                'head' => 'XL WooCommerce Sales Triggers',
                'desc' => 'Use 7 Built-in Sales Triggers to Optimise Single Product Pages For More Conversions',
            );
        }
        if (!class_exists('XLWCTY_Core')) {
            $nextmove_link = add_query_arg(array(
                'utm_source' => 'finale-lite',
                'utm_medium' => 'sidebar',
                'utm_campaign' => 'other-products',
                'utm_term' => 'nextmove',
                    ), 'https://xlplugins.com/woocommerce-thank-you-page-nextmove/');
            $other_products['nextmove'] = array(
                'image' => 'nextmove.png',
                'link' => $nextmove_link,
                'head' => 'NextMove WooCommerce Thank You Pages',
                'desc' => 'Get More Repeat Orders With 17 Plug n Play Components',
            );
        }
        if (is_array($other_products) && count($other_products) > 0) {
            ?>
            <h3>Checkout Our Other Plugins:</h3>
            <?php
            foreach ($other_products as $product_short_name => $product_data) {
                ?>
                <div class="postbox wcct_side_content wcct_xlplugins wcct_xlplugins_<?php echo $product_short_name ?>">
                    <a href="<?php echo $product_data['link'] ?>" target="_blank"></a>
                    <img src="<?php echo plugin_dir_url(WCCT_PLUGIN_FILE) . 'admin/assets/img/' . $product_data['image']; ?>" />
                    <div class="wcct_plugin_head"><?php echo $product_data['head'] ?></div>
                    <div class="wcct_plugin_desc"><?php echo $product_data['desc'] ?></div>
                </div>
                <?php
            }
            ?>
        <?php }
        ?>
        <div class="postbox wcct_side_content">
            <div class="inside">
                <h3>Resources</h3>
                <ul>
                    <li><a href="<?php echo $go_pro_link ?>" target="_blank">Get PRO</a></li>
                    <li><a href="<?php echo $demo_link ?>" target="_blank">Demo</a></li>
                    <li><a href="<?php echo $support_link ?>" target="_blank">Support</a></li>
                    <li><a href="<?php echo $documentation_link ?>" target="_blank">Documentation</a></li>                    
                </ul>
            </div>
        </div>
        <?php
    }

    public function schedule_license_check() {
        wp_schedule_single_event(time() + 10, 'wcct_maybe_schedule_check_license');
    }

    public function modify_uninstall_reason($reasons) {
        $reasons_our = $reasons;


        $reason_other = array(
            'id' => 7,
            'text' => __("Other", WCCT_TEXTDOMAIN),
            'input_type' => 'textfield',
            'input_placeholder' => __("Other", WCCT_TEXTDOMAIN),
        );


        $reasons_our[WCCT_PLUGIN_BASENAME] = array(
            array(
                'id' => 8,
                'text' => __("I am going to upgrade to PRO version", WCCT_TEXTDOMAIN),
                'input_type' => '',
                'input_placeholder' => __("I am going to upgrade to PRO version", WCCT_TEXTDOMAIN),
            ),
            array(
                'id' => 3,
                'text' => XL_deactivate::load_str('reason-needed-for-a-short-period'),
                'input_type' => '',
                'input_placeholder' => XL_deactivate::load_str('reason-needed-for-a-short-period'),
            ),
            array(
                'id' => 4,
                'text' => XL_deactivate::load_str('reason-broke-my-site'),
                'input_type' => '',
                'input_placeholder' => ''
            ),
            array(
                'id' => 5,
                'text' => XL_deactivate::load_str('reason-suddenly-stopped-working'),
                'input_type' => '',
                'input_placeholder' => ''
            ),
            array(
                'id' => 11,
                'text' => __('I was unable to set up the Campaign', WCCT_TEXTDOMAIN),
                'input_type' => '',
                'input_placeholder' => ''
            ),
            array(
                'id' => 12,
                'text' => __('Campaign was running but Countdown Timer didn\'t show', WCCT_TEXTDOMAIN),
                'input_type' => '',
                'input_placeholder' => ''
            ),
            array(
                'id' => 13,
                'text' => __('Campaign was running but Counter Bar didn\'t show', WCCT_TEXTDOMAIN),
                'input_type' => '',
                'input_placeholder' => ''
            ),
            array(
                'id' => 14,
                'text' => __('I set up Discounts but the expected discount amount didn\'t appear', WCCT_TEXTDOMAIN),
                'input_type' => '',
                'input_placeholder' => ''
            ),
            array(
                'id' => 15,
                'text' => __('I set up Inventory but the expected stock units didn\'t appear', WCCT_TEXTDOMAIN),
                'input_type' => '',
                'input_placeholder' => ''
            ),
            array(
                'id' => 16,
                'text' => __('I set up Rules but campaigns were not restricted as per rules', WCCT_TEXTDOMAIN),
                'input_type' => '',
                'input_placeholder' => ''
            ),
        );


        array_push($reasons_our[WCCT_PLUGIN_BASENAME], $reason_other);
        return $reasons_our;
    }

}

if (class_exists("WCCT_XL_Support")) {
    WCCT_Core::register("xl_support", "WCCT_XL_Support");
}
