<?php


/**
 *
 *
 * @author XLPlugins
 * @package XLCore
 */
class XL_Support
{

    protected static $instance;
    public $validation = true;
    /**
     *
     * XL_Support constructor.
     */
    public function __construct()
    {


    }

    /**
     * Creates and instance of the class
     * @return type
     */
    public static function get_instance()
    {


        if (null == self::$instance) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    /**
     * Prepares the system information report.
     *
     * @access public
     * @return string $report
     */
    public function prepare_system_information_report()
    {

        global $wpdb;

        /* Queue up needed information. */
        $php_extensions = get_loaded_extensions();
        $php_ini_get = array('memory_limit', 'max_execution_time', 'upload_max_filesize', 'max_file_uploads', 'post_max_size', 'max_input_vars');
        $mysql_version = $wpdb->get_var($wpdb->prepare("SELECT VERSION() AS %s", 'version'));
        $db_character_set = $wpdb->get_var("SELECT @@character_set_database");
        $db_collation = $wpdb->get_var("SELECT @@collation_database");
        $plugins = get_plugins();
        $active_plugins = get_option('active_plugins', array());
        $active_network_plugins = is_multisite() ? $wpdb->get_var($wpdb->prepare("SELECT meta_value FROM {$wpdb->sitemeta} WHERE meta_key=%s", 'active_sitewide_plugins')) : array();
        $has_log_files = false;

        $theme =  array();
        $get_theme_info = wp_get_theme();
        $theme['name'] = $get_theme_info->get('Name');
        $theme['uri'] = $get_theme_info->get('ThemeURI');
        $theme['version'] = $get_theme_info->get('Version');
        $theme['author'] = $get_theme_info->get('Author');
        $theme['author_uri'] = $get_theme_info->get('AuthorURI');

        /* Begin report. */
        $report = '#### System Information Report ####' . "\n\n";

        /* PHP information. */
        $report .= '*** PHP Information ***' . "\n";
        $report .= 'Version: ' . phpversion() . "\n";
        foreach ($php_ini_get as $ini_get) {
            $report .= $ini_get . ': ' . ini_get($ini_get) . "\n";
        }
        $report .= 'cURL Enabled: ' . (function_exists('curl_init') ? 'Yes' : 'No') . "\n";
        $report .= 'Mcrypt Enabled: ' . (function_exists('mcrypt_encrypt') ? 'Yes' : 'No') . "\n";
        $report .= 'Mbstring Enabled: ' . (function_exists('mb_strlen') ? 'Yes' : 'No') . "\n";
        $report .= 'Loaded Extensions: ' . implode(', ', $php_extensions) . "\n\n";

        /* MySQL information. */
        $report .= '*** MySQL Information ***' . "\n";
        $report .= 'Version: ' . $mysql_version . "\n";
        $report .= 'Database Character Set: ' . $db_character_set . "\n";
        $report .= 'Database Collation: ' . $db_collation . "\n\n";

        /* Web server information. */
        $report .= '*** Web Server Information ***' . "\n";
        $report .= 'Software: ' . $_SERVER['SERVER_SOFTWARE'] . "\n";
        $report .= 'Port: ' . $_SERVER['SERVER_PORT'] . "\n";
        $report .= 'Document Root: ' . $_SERVER['DOCUMENT_ROOT'] . "\n\n";

        /* WordPress information. */
        $report .= '*** WordPress Information ***' . "\n";
        $report .= 'WP_DEBUG: ' . (WP_DEBUG ? 'Enabled' : 'Disabled') . "\n";
        $report .= 'WP_DEBUG_LOG: ' . (WP_DEBUG_LOG ? 'Enabled' : 'Disabled') . "\n";
        $report .= 'WP_MEMORY_LIMIT: ' . WP_MEMORY_LIMIT . "\n";
        $report .= 'Multisite: ' . (is_multisite() ? 'Enabled' : 'Disabled') . "\n";
        $report .= 'Site URL: ' . get_site_url() . "\n";
        $report .= 'Home URL: ' . get_home_url() . "\n\n";


        /* Current Theme Info. */
        $report .= '*** Theme Information ***' . "\n";
        $report .= 'Name: ' . $theme['name'] . "\n";
        $report .= 'URI: ' . $theme['uri'] . "\n";
        $report .= 'Version: ' . $theme['version'] . "\n";
        $report .= 'Author: ' . $theme['author'] . "\n";
        $report .= 'Author URI: ' . $theme['author_uri'] . "\n";



        /* Plugin information. */
        $report .= '*** Active Plugins ***' . "\n";
        foreach ($plugins as $plugin_path => $plugin) {
            if (in_array($plugin_path, $active_plugins)) {
                $report .= $plugin['Name'] . ' (' . $plugin['Version'] . ')' . "\n";
            }
        }
        $report .= "\n";

        /* Multisite plugin information. */
        if (is_multisite() && !empty($active_network_plugins)) {
            $report .= '*** Network Active Plugins ***' . "\n";
            $active_network_plugins = maybe_unserialize($active_network_plugins);
            foreach ($active_network_plugins as $plugin_slug => $activated) {
                $plugin = get_plugin_data(WP_CONTENT_DIR . '/plugins/' . $plugin_slug);
                $report .= $plugin['Name'] . ' (' . $plugin['Version'] . ')' . "\n";
            }
            $report .= "\n";
        }


        $report .= "\n" . '#### System Information Report ####';

        return $report;

    }


    /**
     * Processing support request
     * @param $posted_data
     * @uses XL_API used to fire api request to generate request
     * @uses XL_admin_notifications pushing success and failure notifications
     * @since 1.0.4
     */
    public function xl_maybe_push_support_request($posted_data)
    {

        if(!$this->validation){
            return;
        }
        wp_verify_nonce($posted_data['nonce']);

        $response = XL_API::post_support_request($posted_data);

        if ($response->status == "success") {
            XL_admin_notifications::add_notification( array( 'support_request_success' => array(
                'type' => 'success',
                'is_dismissable' => true,
                'content' => __('<p> Your Support Request has been submitted successfully. We will contact you shortly. Thank You!</p>', 'xlplugins'),

            ) ) );
        } else {
            XL_admin_notifications::add_notification(array('support_request_failure' => array(
                'type' => 'error',
                'is_dismissable' => true,
                'content' => __('<p> Unable to submit your request at this moment. Please Try Again Later.</p>', 'xlplugins'),

            )));
        }


    }


//put your code here
}
